#!/bin/bash

function usage
{
	echo 'ACME Clallenge LFTP Uploader v1.0.0 [2021/03/05]'
	echo '(C) 2021 Evgeny Vrublevsky <me@veg.by>'
	echo 'Usage:'
	echo '    acmechal.sh <help|auth|cleanup> <ftp://user:pass@host/path/to/site/root/>'
	echo 'Options:'
	echo '    --mode <auth|cleanup>                     Set CERTBOT_MODE.'
	echo '    --sitepath <ftp://user:pass@host/path/>   Set CERTBOT_SITEPATH.'
	echo '    --token <filename>                        Set CERTBOT_TOKEN.'
	echo '    --validation <content>                    Set CERTBOT_VALIDATION.'
}

if [[ -z "$1" ]] || [[ "$1" = "-h" ]] || [[ "$1" = "--help" ]] || [[ "$1" = "help" ]]; then
	usage
	exit 0
fi

# Parse arguments.

POSITIONAL=()
while [[ $# -gt 0 ]]; do
case "$1" in
	--mode)
		CERTBOT_MODE="$2"
		shift
		shift
		;;
	--sitepath)
		CERTBOT_SITEPATH="$2"
		shift
		shift
		;;
	--token)
		CERTBOT_TOKEN="$2"
		shift
		shift
		;;
	--validation)
		CERTBOT_VALIDATION="$2"
		shift
		shift
		;;
	*)
		# Unknown option.
		POSITIONAL+=("$1")
		shift
		;;
esac
done

# Parse positional parameters.

set -- "${POSITIONAL[@]}"

if [[ -z "${CERTBOT_MODE}" ]]; then
	CERTBOT_MODE="$1"
fi
if [[ -z "${CERTBOT_SITEPATH}" ]]; then
	CERTBOT_SITEPATH="$2"
fi

# Check if required variables are provided.

if [[ "${CERTBOT_MODE}" != "auth" ]] && [[ "${CERTBOT_MODE}" != "cleanup" ]]; then
	echo 'Error: CERTBOT_MODE should be "auth" or "cleanup".' 1>&2
	exit 1
fi
if [[ -z "${CERTBOT_TOKEN}" ]] || [[ -z "${CERTBOT_SITEPATH}" ]]; then
	echo 'Error: CERTBOT_TOKEN and CERTBOT_SITEPATH are required.' 1>&2
	exit 1
fi

# Validate values.

if [[ ! $CERTBOT_TOKEN =~ ^[^/]+$ ]]; then
	echo 'Error: CERTBOT_TOKEN is invalid.' 1>&2
	exit 1
fi

# Execute LFTP according to current mode.

RESULT=0

if [[ "${CERTBOT_MODE}" = "auth" ]]; then

	echo -n "$CERTBOT_VALIDATION" > /tmp/$CERTBOT_TOKEN || exit 1
	lftp <<EOL
set ftp:ssl-allow false
set ssl:verify-certificate false
set ssl:check-hostname false
set net:timeout 5
set net:max-retries 1
open ${CERTBOT_SITEPATH} || quit 1
mkdir -p -f .well-known/acme-challenge
cd .well-known/acme-challenge || quit 1
put -e /tmp/$CERTBOT_TOKEN || quit 1
quit 0
EOL
	RESULT=$?
	rm /tmp/$CERTBOT_TOKEN

else

	lftp <<EOL
set ftp:ssl-allow false
set ssl:verify-certificate false
set ssl:check-hostname false
set net:timeout 5
set net:max-retries 1
open ${CERTBOT_SITEPATH} || quit 1
cd .well-known/acme-challenge || quit 1
rm $CERTBOT_TOKEN || quit 1
glob -a --exist * && quit 0
cd .. || quit 0
rmdir acme-challenge || quit 0
glob -a --exist * && quit 0
cd .. || quit 0
rmdir .well-known
quit 0
EOL
	RESULT=$?

fi

exit $RESULT
