#!/bin/bash

function usage
{
	echo 'Certbot Manual Site Adder v1.0.0 [2021/03/05]'
	echo '(C) 2021 Evgeny Vrublevsky <me@veg.by>'
	echo 'Usage:'
	echo '    ./addnew.sh --sitepath <ftp://user:pass@host/path/> <main.host> <alias.host>'
}

if [[ -z "$1" ]] || [[ "$1" = "-h" ]] || [[ "$1" = "--help" ]] || [[ "$1" = "help" ]]; then
	usage
	exit 0
fi

# Parse arguments.

POSITIONAL=()
while [[ $# -gt 0 ]]; do
case "$1" in
	--sitepath)
		CERTBOT_SITEPATH="$2"
		shift
		shift
		;;
	--domains)
		CERTBOT_DOMAINS="$2"
		shift
		shift
		;;
	*)
		# Unknown option.
		POSITIONAL+=("$1")
		shift
		;;
esac
done

# Parse positional parameters.

set -- "${POSITIONAL[@]}"

if [[ -z "${CERTBOT_DOMAINS}" ]]; then
	CERTBOT_DOMAINS="$(echo "$*" | tr " " ,)"
fi
if [[ -z "${CERTBOT_SITEPATH}" ]]; then
	# Default CERTBOT_SITEPATH value.
	# CERTBOT_SITEPATH="ftp://ftpuser@ftp.example.com/$1/docs/"
fi

# Check if required variables are provided.

if [[ -z "${CERTBOT_DOMAINS}" ]] || [[ -z "${CERTBOT_SITEPATH}" ]]; then
	echo 'Error: CERTBOT_DOMAINS and CERTBOT_SITEPATH are required.' 1>&2
	exit 1
fi

AUTH_HOOK="/etc/letsencrypt/scripts/acmelftp.sh auth '${CERTBOT_SITEPATH}'"
CLEANUP_HOOK="/etc/letsencrypt/scripts/acmelftp.sh cleanup '${CERTBOT_SITEPATH}'"

certbot certonly --manual --manual-public-ip-logging-ok --preferred-challenges=http-01 --manual-auth-hook "${AUTH_HOOK}" --manual-cleanup-hook "${CLEANUP_HOOK}" --domains "${CERTBOT_DOMAINS}"
